//
// Make needle image given motion vectors in format written by motionblocks
// John Robinson
//
#include "usage.h"
#include "picture.h"
#include <fstream>
#include <stdlib.h>
#include <math.h>
int blocksize;
int needlevalue;
int drawneedle(picture_of_int& needle, int i, int j, int fromi, int fromj,
							int value)
// Simple line drawing. Puts needle head at location (i,j) then tail drawn
// to (fromi, fromj).
	{
	needle[i][j] = value;
	needle[i-1][j]=needle[i+1][j] = needle[i][j-1] = needle[i][j+1] = value;
	// Simple line drawing algorithm
	if (abs(fromj) > abs(fromi)) {
		float m = (float)fromi/fromj;
		float c = (float)i - m*j;
		int xinc = (fromj < 0) ? -1 : 1;
		for (int x = j; x != j+fromj; x += xinc)
			needle[(int)(m*x+c+0.5)][x] = value;
		needle[(int)(m*(j+fromj)+c+0.5)][(j+fromj)] = value;
		}
	else {
		if (fromi == 0)		// 0 vector, don't draw needle
			return 0;
		float m = (float)fromj/fromi;
		float c = (float)j - m*i;
		int yinc = (fromi < 0) ? -1 : 1;
		for (int y = i; y != i+fromi; y += yinc)
			needle[y][(int)(m*y+c+0.5)] = value;
		needle[i+fromi][(int)(m*(i+fromi)+c+0.5)] = value;
		}
	return 0;
	}

int main(int argc, char *argv[])
	{
	usage("needle currframe motionvecs needlepic needlevalue");
	colour_picture in(argv[1]);
	int ymax = in.nrows();
	int xmax = in.ncols();
	colour_picture needle(in);
	ifstream vecs(argv[2]);
	// ignore blocksize and maximum displacement values
	int dummy;
	vecs >> dummy;
	vecs >> dummy;
	int value = atoi(argv[4]);
	while(!vecs.eof()) {
		int i, j, ivec, jvec;
		vecs >> i >> j >> ivec >> jvec;
		if (in.ismonochrome())
			drawneedle(needle.mono, i, j, ivec, jvec, value);
		else {
			drawneedle(needle.red, i, j, ivec, jvec, value);
			drawneedle(needle.green, i, j, ivec, jvec, value);
			drawneedle(needle.blue, i, j, ivec, jvec, value);
			}
		}
	needle.write(argv[3]);
	return 1;
	}
