//
// One-dimensional predictive ddeecoder
// John Robinson, January 1997
//
#include <stdlib.h>
#include <limits.h>
#include "picture.h"

//
// The quantizer stuff is copied directly from pred1d.C. Should really
// have separate files quantizer.C, quantizer.h, that are shared by the
// encoder and decoder. Done this way for ease of reading.
//
const int numlevels = 16;
const float linear[numlevels/2] =  {0.5,1.5,2.5,3.5,4.5,5.5,6.5,7.5};
const float tapered[numlevels/2] =  {0.5,1.5,3.5,6.5,10.5,16.5,28.5,50.5};
//
// numlevels/2 is used because the quantizer will be symmetric about 0.
// Indeed, it will be zero-step centred, and therefore use an odd number of
// levels, numlevels-1.
//
class dpcm_quantizer {
//
// Implements a dpcm quantizer by way of lookup tables. Instantiation with
// a scaling factor and quantizer type causes creation of the encode and
// decode lookup tables from the elementary arrays linear and tapered.
// Encoding and decoding are then done by simple, inlined table lookups.
//
// This quantizer outputs numlevels different values, with numlevels/2
// representing a prediction error of 0.
//
        int Aencode[511];
        int *encode;
        // encode will function like a zero-centred array
        int decode[numlevels];
public:
        dpcm_quantizer(int scale, int type) {
                encode = &Aencode[255];
                // encode points to the centre of the arrays
                const float *p;
                if (type == 0)          // Linear
                        p = linear;
                else
                        p = tapered;
                int cnt = 0;
                int replevel = 0;
                // Representative levels will simply be the mean of the
                // two surrounding decision levels
                decode[numlevels/2+cnt] = replevel;
                // I.e. decode[numlevels/2] = 0;
                for (int i = 0; i < 256; i++) {
                        if ((i >= *p * scale)&&(cnt < numlevels/2-1)) {
                                replevel = (int)(((*(p+1)+*p)*scale + 1)/2);
                                cnt++;
                                decode[numlevels/2+cnt] = replevel;
                                decode[numlevels/2-cnt] = -replevel;
                                p++;
                                }
                        encode[i] = numlevels/2 + cnt;
                        encode[-i] = numlevels/2 - cnt;
                        }
                }
        int enc(int value) { return encode[value]; }
        int dec(int value) { return decode[value]; }
        };


int main(int argc, char *argv[])
	{
        if (argc != 5) {
                cerr << "Usage: unpred1d diffspic outpic quant_type quant_scale";
                cerr << "\nUse same values for quant_type and quant_scale ";
                cerr << "as used in encoding with pred1d\n";
                return -1;
                }
	// Note that quant_type and quant_scale must be same as used as
	// encoder. In a real system, the type and scale of the quantizer
	// would be fixed, or else information about the quantizer would
	// be included as overhead data in the compressed file.
	picture_of_int in(argv[1]);
	int type = atoi(argv[3]);
	int scale = atoi(argv[4]);
	dpcm_quantizer q(scale, type);
	int prev = 0;		// Value of previous pel
	for (int i = 0; i < in.nrows(); i++)
		{
		prev = 0;
		for (int j = 0; j < in.ncols(); j++)
			{
			prev = in[i][j] = prev + q.dec(in[i][j]);
			if (prev < 0)
				in[i][j] = prev = 0;
			if (prev > 255)
				in[i][j] = prev = 255;
			}
		}
	in.write(argv[2]);
	return 1;
	}

