// Program to demonstrate some of the features of irange
#include "picture.h"
// Some callbacks used in the demonstration
int thresh1(int& pel) {
	if (pel < 128) pel = 0;
	else pel = 255;
	return 1;
	}
int thresh2(int& outpel, const int& inpel) {
	if (inpel < 128) outpel = 0;
	else outpel = 255;
	return 1;
	}
int average(int& outpel, const int **buf) {
	outpel = buf[-1][-1] + buf[-1][0] + buf[-1][1] +
		 buf[0][-1]  + buf[0][0]  + buf[0][1] +
		 buf[1][-1]  + buf[1][0]  + buf[1][1];
	outpel /= 9;
	return 1;
	}
int main(int argc, char *argv[]) {
	if (argc != 1) {
		cerr << "Usage: irangedemo";
		exit(1);
		}
	colour_picture pic("../pics/baboon512");
	colour_picture other("../pics/lenna512");
	irange basic(0,1,127);	// Basic block size
	irange evensamples(0,2,254);	// For demos with subsampling
	irange rowrange(basic);
	irange colrange(basic);
	irange otherrowrange(basic);
	irange othercolrange(basic);
	// First row of four blocks demos operations with integers
	// Leave very first block alone: Don't do anything
	
	// For second block, set whole block equal to 50
	colrange += 128;
	pic[rowrange][colrange] = 50;
	// Invert third block
	colrange += 128;
	pic[rowrange][colrange] *= -1;
	pic[rowrange][colrange] += 256;	// Move back into [0,255] range
	// Halve amplitude of fourth block
	colrange += 128;
	pic[rowrange][colrange] /= 2;

	// Second row of four blocks demos arithmetic operations with
	// another picture
	// First block is just copy from (top left block of) other picture
	colrange = basic;	// Reset colrange to (0,1,127)
	rowrange += 128;
	pic[rowrange][colrange] = other[otherrowrange][othercolrange];
	// Second block is copy from elsewhere in other picture
	colrange += 128;
	pic[rowrange][colrange] = other[200+otherrowrange][200+othercolrange];
	// (Note how adding an int to an irange yields a shifted irange)

	// Third block is average of block at this location in two pictures
	colrange += 128;
	pic[rowrange][colrange] += other[rowrange][colrange];
	pic[rowrange][colrange] /= 2;
	// Fourth block is difference between two blocks (plus an offset)
	colrange += 128;
	pic[rowrange][colrange] -= other[rowrange][colrange];
	pic[rowrange][colrange] += 128; // To give vals mostly in [0,255].

	// Third row of four blocks demos the three non-arithmetic operators
	// and iranges with different spacings.
	colrange = basic;
	rowrange += 128;
	// The operator |= produces the absolute difference between blocks
	pic[rowrange][colrange] |= other[rowrange][colrange];

	// The operator ^= produces the squared difference between blocks
	colrange += 128;
	pic[rowrange][colrange] ^= other[rowrange][colrange];
	// Have to scale result or else whole picture will display poorly
	int blockmax = pic[rowrange][colrange].max(); // NOTE: max() applies
						      // just to specified
						      // block.
	pic[rowrange][colrange] *= 255;
	pic[rowrange][colrange] /= blockmax;

	// The operator &= does conditional replacement. If the pel in
	// the source picture is greater than or equal to 0, it replaces
	// the destination picture. If less than 0, the source pel
	// remains.
	// To illustrate this, we need to create a masked version of part
	// of the other picture
	colour_picture temppic(128,128);
	temppic = other[rowrange][colrange];
	// Let's just preserve a disk of temppic, by putting negative values
	// into the pels outside a disk.
	const int radius = 50;
	for (int i = 0; i < 128; i++)
	for (int j = 0; j < 128; j++) {
		int x = j - 64;	// Relative to centre of pic
		int y = i - 64;
		if (x*x + y*y > radius*radius)
			temppic[i][j] = -1;
		}
	// Now, &= should do appropriate masking
	colrange += 128;
	pic[rowrange][colrange] &= temppic;

	// Using the fourth block on the third row, we demonstrate 
	// subsampling a source
	// block and putting it in a destination block
	colrange += 128;
	pic[rowrange][colrange] = other[evensamples][evensamples];
	// Note that although this is the only example where the other
	// samples have a different spacing, all the operators illustrated
	// here work no matter what the irange spacings. What counts is
	// that the number of integers in the irange sequences are the same.

	
	// The final row of four blocks demos member functions applied just
	// to blocks.
	// Already seen max() member function. min() works similarly
	// Write a block of the picture to a file
	colrange = basic;
	rowrange += 128;
	pic[rowrange][colrange].write("temppic");

	// Apply a point operator to output picture
	colrange += 128;
	pic[rowrange][colrange].point(thresh1);
	// Apply a point operator to other picture and show in output
	colrange += 128;
	temppic = other[otherrowrange][othercolrange];
	pic[rowrange][colrange].point(temppic,thresh2);
	// Apply a neigh operator to other picture and show in output
	colrange += 128;
	pic[rowrange][colrange].neigh(temppic,average);
	pic.show();
	pic.inspect();
	return 1;
	}

